package core

// Scope represents a map of variables.
type Scope map[string]*Variable

// Scope returns the current scope.
func (s *state) Scope() Scope {
	return s.scopes[len(s.scopes)-1]
}

// pushScope pushes a new scope to the top of the stack.
func (s *state) pushScope() {
	lastScope := s.scopes[len(s.scopes)-1]
	newScope := make(Scope, len(lastScope))

	for k, v := range lastScope {
		newScope[k] = &Variable{
			Value:    v.Value,
			Name:     v.Name,
			Register: v.Register,
			Alive:    v.Alive,
		}
	}

	s.scopes = append(s.scopes, newScope)
}

// popScope removes the scope at the top of the stack.
func (s *state) popScope() {
	s.scopes = s.scopes[:len(s.scopes)-1]
}
